/***************************************************************************
                          dms.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Sun Feb 11 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include <qstring.h>
#include "skypoint.h"
#include "dms.h"

dms::dms(int d, int m, int s)
{
  Deg = d; Min = m; Sec = s;
  D = (double)abs(d) + ((double)m + (double)s/60.)/60.;
  if (d<0) { D = -1.0*D; }
}

dms::dms( double x )
{
  D = x;
  double ax = fabs(x);

  Deg = int(ax);
  Min = int(60.0*(ax-Deg));
  Sec = int(60.0*(60.0*(ax-Deg)-Min));
  if (x<0) {
    if ( Deg == 0 && Min == 0 ) {
			Sec *= -1;
    } else if ( Deg == 0 ) {
      Min *= -1;
    } else {
      Deg *= -1;
    }
  }
}

void dms::setD( double x )
{
  D = x;
  double ax = fabs(x);

  Deg = int(ax);
  Min = int(60.0*(ax-Deg));
  Sec = int(60.0*(60.0*(ax-Deg)-Min));
  if (x<0) {
    if ( Deg == 0 && Min == 0 ) {
      Sec *= -1;
    } else if ( Deg == 0 ) {
      Min *= -1;
    } else {
      Deg *= -1;
    }
  }
}

void dms::setD(int d, int m, int s)
{
  Deg = d; Min = m; Sec = s;
  D = (double)abs(d) + ((double)m + (double)s/60.)/60.;
  if (d<0) {D = -1.0*D;}
}

dms dms::operator+ (dms &angle)
{
  int d(0),m(0),s(0);

  s = Sec + angle.getSec();
  while (s >= 60)
  {
  	s -= 60;
    m++;
  }

  m += Min + angle.getMin();
  while (m >= 60)
  {
    m -= 60;
    d++;
  }

  d += Deg + angle.getDeg();
  while (d >=24) { d -= 24; }

  return dms(d,m,s);
}

dms dms::operator- (dms &angle)
{
  int d(0),m(0),s(0);

  s = Sec - angle.getSec();
  while (s < 0)
  {
    s += 60;
    m--;
  }

  m += Min - angle.getMin();
  while (m < 0)
  {
    m += 60;
    d--;
  }

  d += Deg - angle.getDeg();
  while (d < 0) { d += 24; }

  return dms(d,m,s);
}

//---------------------------------------------------------------------------

double PI()
{
  double PI = acos(-1.0);
  return PI;
}
//---------------------------------------------------------------------------

void SinCos(double angle, double &sina, double &cosa)
{
	double dangle = DegToRad(angle);
  sina = sin(dangle);
	cosa = cos(dangle);
}
//---------------------------------------------------------------------------

double HourToDeg(double Hour)
{
  //if (Hour>12) {Hour -= 24.0;}
  double Deg = 15.0*Hour;
  return Deg;
}
//---------------------------------------------------------------------------

double DegToHour(double Deg)
{
  if (Deg<0) {Deg += 360.0;}
  double Hour = Deg/15.0;
  return Hour;
}
//---------------------------------------------------------------------------

double DegToRad(double Deg)
{
  if (Deg<0) {Deg += 360.0;}
  double Rad = Deg*PI()/180.0;
  return Rad;
}
//---------------------------------------------------------------------------

double RadToDeg(double Rad)
{
  //if (Rad>PI()) {Rad -= 2.0*PI();}
  double Deg = Rad*180.0/PI();
  return Deg;
}
//---------------------------------------------------------------------------

double HourToRad(double Hour)
{
  double Rad = Hour*15.0*PI()/180.0;
  return Rad;
}
//---------------------------------------------------------------------------

double RadToHour(double Rad)
{
  double Hour = Rad*180.0/PI()/15.0;
	while (Hour < 0.0) Hour += 24.0;
	while (Hour > 24.0) Hour -= 24.0;
  return Hour;
}
//---------------------------------------------------------------------------

void dXdYToRaDec( double dx, double dy, SkyPoint focus, bool useAltAz, double LSTh, double lat, double &RA, double &Dec ) {

	double sinDec, cosDec, sinDec0, cosDec0, sinc, cosc, sinlat, coslat;
	double xx, yy, A;

	double r  = sqrt( dx*dx + dy*dy );
	double c = 2.0*asin(0.5*r);

	SinCos( focus.getDec(), sinDec0, cosDec0 );
	SinCos( RadToDeg(c), sinc, cosc );

	if ( useAltAz ) {
		double alt, az, alt0, az0, HA;
		double sinAlt, cosAlt, sinAlt0, cosAlt0, sinAz, cosAz;
		double HA0 = LSTh - focus.getRA();
//		HADecToAltAz( HA0, focus.getDec(), lat, az0, alt0 );
		az0 = focus.getAz();
		alt0 = focus.getAlt();
		SinCos( alt0, sinAlt0, cosAlt0 );

		dx = -dx; //Flip East-west (Az goes in opposite direction of RA)
		yy = dx*sinc;
		xx = r*cosAlt0*cosc - dy*sinAlt0*sinc;
		A = RadToDeg( atan( yy/xx ) );

		//resolve ambiguity of atan():
		if ( xx<0 ) A = A + 180.0;
		if ( xx>0 && yy<0 ) A = A + 360.0;

		az = az0 + A;
		alt = RadToDeg( asin( cosc*sinAlt0 + ( dy*sinc*cosAlt0 )/r ) );
		SinCos( az, sinAz, cosAz );
		SinCos( alt, sinAlt, cosAlt );
		SinCos( lat, sinlat, coslat );

		Dec = RadToDeg( asin( sinAlt*sinlat + cosAlt*coslat*cosAz ) );
		SinCos( Dec, sinDec, cosDec );
		HA =  acos( ( sinAlt - sinlat*sinDec )/( coslat*cosDec ) );
		if ( sinAz > 0.0 ) HA = 2.0*PI() - HA;
		RA = LSTh - RadToHour( HA );

  } else {
		yy = dx*sinc;
		xx = r*cosDec0*cosc - dy*sinDec0*sinc;
		A = RadToDeg( atan( yy / xx ) );

		//resolve ambiguity of atan():
		if ( xx<0 ) A = A + 180.0;
		if ( xx>0 && yy<0 ) A = A + 360.0;

		RA  = focus.getRA() + DegToHour( A );
		Dec = RadToDeg( asin( cosc*sinDec0 + (dy*sinc*cosDec0)/r ) );
	}
}
//---------------------------------------------------------------------------

void precess( double RA0, double Dec0, double &RA, double &Dec, double JD, double JD0 ) {
	double T, X, Z, Y;
	double CX, SX, CY, SY, CZ, SZ;
	double cosRA, sinRA, cosDec, sinDec;
	double P[3][3];
	double v[3], s[3];

	SinCos( RA0, sinRA, cosRA );
	SinCos( Dec0, sinDec, cosDec );

	if ( JD0 != 2451545.0 ) { //Need to first precess to J2000.0 coords
		T = (JD0 - 2451545.0)/36525.0; // T is number of centuries from J2000.0

		X = 0.6406161*T + 0.0000839*T*T + 0.0000050*T*T*T;
		Z = 0.6406161*T + 0.0003041*T*T + 0.0000051*T*T*T;
		Y = 0.5567530*T - 0.0001185*T*T - 0.0000116*T*T*T;

		SinCos( X, SX, CX );
		SinCos( Y, SY, CY );
		SinCos( Z, SZ, CZ );

		P[0][0] = CX*CY*CZ - SX*SZ;      //P is an array that handles the precession
		P[1][0] = CX*CY*SZ + SX*CZ;
		P[2][0] = CX*SY;
		P[0][1] = -1.0*SX*CY*CZ - CX*SZ;
		P[1][1] = -1.0*SX*CY*SZ - CX*CZ;
		P[2][1] = -1.0*SX*SY;
		P[0][2] = -1.0*SY*CZ;
		P[1][2] = -1.0*SY*SZ;
		P[2][2] = CY;

		v[0] = cosRA*cosDec; //v is a column vector representing input coordinates.
		v[1] = sinRA*cosDec;
		v[2] = sinDec;	

		//s is the product of P and v; s represents the coordinates precessed to J2000
		for ( unsigned int i=0; i<3; ++i ) {
			s[i] = P[0][i]*v[0] + P[1][i]*v[1] + P[2][i]*v[2];
		}

	} else { //Input coords already in J2000, set s accordingly.
		s[0] = cosRA*cosDec;
		s[1] = sinRA*cosDec;
		s[2] = sinDec;	
  }

	//Now, precess from J2000 to desired epoch (JD).
	T = (JD - 2451545.0)/36525.0; // T is number of centuries from J2000.0

	X = 0.6406161*T + 0.0000839*T*T + 0.0000050*T*T*T;
	Z = 0.6406161*T + 0.0003041*T*T + 0.0000051*T*T*T;
	Y = 0.5567530*T - 0.0001185*T*T - 0.0000116*T*T*T;

	SinCos( X, SX, CX );
	SinCos( Y, SY, CY );
	SinCos( Z, SZ, CZ );

  //P is now the *transpose* of the above matrix, because we are precessing from
  //(rather than to) J2000.
	P[0][0] = CX*CY*CZ - SX*SZ;
	P[1][0] = -1.0*SX*CY*CZ - CX*SZ;
	P[2][0] = -1.0*SY*CZ;
	P[0][1] = CX*CY*SZ + SX*CZ;
	P[1][1] = -1.0*SX*CY*SZ - CX*CZ;
	P[2][1] = -1.0*SY*SZ;
	P[0][2] = CX*SY;
	P[1][2] = -1.0*SX*SY;
	P[2][2] = CY;

	//Multiply P and s to get v, the vector representing the new coords.
	for ( unsigned int i=0; i<3; ++i ) {
		v[i] = P[0][i]*s[0] + P[1][i]*s[1] + P[2][i]*s[2];
	}

	//Extract RA, Dec from the vector:
	RA = atan( v[1]/v[0] );
	Dec = asin( v[2] );

	if ( v[0] < 0.0 ) {
		RA = RA + 180.0;
	} else if( v[1] < 0.0 ) {
		RA = RA + 360.0;
	}
}

void nutate( double JD, double &dEcLong, double &dObliq ) {
	//This uses the pre-1984 theory of nutation.  The results
	//are accurate to 0.5 arcsec, should be fine for KStars.
	double T, L, O, A, B;
	double sin2L, cos2L, sinO, cosO;
	int iA, iB;

	T = ( JD - 2415020.0 )/36525.0; //centuries since 1900.0
	A = 100.002136*T;
	B = 5.372617*T;
	iA = int(A);
	iB = int(B);
	
	L = 279.6967 + 360.0*( A - (double)iA ); //mean ecl. long. of Sun
	O = 259.1833 - 360.0*( B - (double)iB ); //ecl. long. of Moon's ascending node

	SinCos( 2.0*L, sin2L, cos2L );
	SinCos( O, sinO, cosO );

	dEcLong = ( -17.2*sinO - 1.3*sin2L )/3600.0; //Ecl. long. correction, in degrees
	dObliq  = (   9.2*cosO + 0.5*cos2L )/3600.0; //Obliq. correction, in degrees
}

