/***************************************************************************
                          finddialog.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Wed Jul 4 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <qframe.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlist.h>
#include <qlistbox.h>
#include <qcombobox.h>
#include <qpushbutton.h>

#include "finddialog.h"
#include "kstars.h"

FindDialog::FindDialog( QWidget* parent )
    : KDialogBase( KDialogBase::Plain, i18n( "Find Object" ), Ok|Apply|Cancel, Apply, parent ) {
	QFrame *page = plainPage();

//Create Layout managers
	vlay = new QVBoxLayout( page, 2, 2 );
	hlay = new QHBoxLayout( 2 ); //this mgr will be added to vlay

//Create Widgets
	SearchBox = new QLineEdit( page, "SearchBox" );
	
	filterTypeLabel = new QLabel( page, "filterTypeLabel" );
	filterTypeLabel->setAlignment( AlignRight );
	filterTypeLabel->setText( "Filter by type: " );
	
	filterType = new QComboBox( page, "filterType" );
	filterType->setEditable( false );
	filterType->insertItem( "Any" );
	filterType->insertItem( "Constellations" );
	filterType->insertItem( "Stars" );
	filterType->insertItem( "Double Stars" );
	filterType->insertItem( "Planets" );
	filterType->insertItem( "Open Clusters" );
	filterType->insertItem( "Glob. Clusters" );
	filterType->insertItem( "Gas. Nebulae" );
	filterType->insertItem( "Plan. Nebulae" );
	filterType->insertItem( "SN Remnants" );
	filterType->insertItem( "Galaxies" );
	
	SearchList = new QListBox( page, "SearchList" );
	SearchList->setMinimumWidth( 256 );
  SearchList->setVScrollBarMode( QListBox::AlwaysOn );
  SearchList->setHScrollBarMode( QListBox::AlwaysOff );

//Pack Widgets into layout manager
	hlay->addWidget( filterTypeLabel, 0, 0 );
	hlay->addWidget( filterType, 0, 0 );
	
	vlay->addWidget( SearchBox, 0, 0 );
	vlay->addSpacing( 12 );
	vlay->addWidget( SearchList, 0, 0 );
	vlay->addLayout( hlay, 0 );
	
	vlay->activate();

  connect( this, SIGNAL( okClicked() ), this, SLOT( accept() ) ) ;
  connect( this, SIGNAL( cancelClicked() ), this, SLOT( reject() ) );
	connect( this, SIGNAL( apply() ), this, SLOT( filter() ) );
//	connect( SearchBox, SIGNAL( textChanged( const QString &s ) ), this, SLOT( filter( const QString &s ) ) );
	connect( filterType, SIGNAL( activated( int ) ), this, SLOT( filterByType( int ) ) );
	
	objNames = new QStringList();
	initObjectList();
}

FindDialog::~FindDialog() {
}

void FindDialog::initObjectList( void ) {
  KStars *p = (KStars *)parent();
	
  //Add Constellations to list
  for ( unsigned int i=0; i< p->cnameList.count(); ++i ) {
    QString s( p->cnameList.at(i)->name );
    objNames->append( s );
		objList.append( p->cnameList.at(i) );
  }

  //Add Messier Objects to list
  for ( unsigned int i=0; i< p->messList.count(); ++i ) {
    QString s( p->messList.at(i)->name );
    objNames->append( s );
    objList.append( p->messList.at(i) );
    QString s2( p->messList.at(i)->longname );
    if (s2.length() > 0) {
    	objNames->append( s2 );
	    objList.append( p->messList.at(i) );
	  }
  }

  //Add NGC Objects to list
  for ( unsigned int i=0; i< p->ngcList.count(); ++i ) {
    QString s( p->ngcList.at(i)->name );
    objNames->append( s );
    objList.append( p->ngcList.at(i) );
    QString s2( p->ngcList.at(i)->longname );
    if (s2.length() > 0) {
    	objNames->append( s2 );
 	   	objList.append( p->ngcList.at(i) );
 	  }
  }

  //Add IC Objects to list
  for ( unsigned int i=0; i< p->icList.count(); ++i ) {
    QString s( p->icList.at(i)->name );
    objNames->append( s );
    objList.append( p->icList.at(i) );
    QString s2( p->icList.at(i)->longname );
    if (s2.length() > 0) {
    	objNames->append( s2 );
	    objList.append( p->icList.at(i) );
    }	
  }

  //Fill the SearchBox with the full list of strings
	SearchBox->clear();
  for ( unsigned int i=0; i<objNames->count(); ++i ) {
	  SearchList->insertItem( (QString)*objNames->at(i) );
	}
}

void FindDialog::filter() {
	SearchList->clear();
	filteredList.clear();
	filterType->setCurrentItem(0);
		
	for ( unsigned int i=0; i<objNames->count(); ++i ) {
		QString s = ((QString) *objNames->at(i));
		QString s2 = s.lower();
		if ( s2.startsWith( SearchBox->text().lower() ) ) {
			SearchList->insertItem( s );
			filteredList.append( (SkyObject *)objList.at(i) );
		}
	}			
}

void FindDialog::filterByType( int iType ) {
	SearchList->clear();
	filteredList.clear();
	SearchBox->clear();
		
	switch ( iType ) {
		case 0: //Any
		  for ( unsigned int i=0; i<objList.count(); ++i ) {
	  		SearchList->insertItem( (QString)*objNames->at(i) );
				filteredList.append( objList.at(i) );
			}
			break;
		case 2: // stars
			//temporarily disabled, pending addition of named stars
			break;
		default: //All others, filter according to objType
			for( unsigned int i=0; i<objList.count(); ++i ) {
					
				if ( objList.at(i)->type + 2 == iType ) {
					SearchList->insertItem( (QString)*objNames->at(i) );
					filteredList.append( objList.at(i) );
				}
			}
			break;
	}
}

