/***************************************************************************
                          kstars.h  -  K Desktop Planetarium
                             -------------------
    begin                : Mon Feb  5 01:11:45 PST 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/




#ifndef KSTARS_H
#define KSTARS_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <kmainwindow.h>
#include <kmenubar.h>
#include <kstddirs.h>
#include <qlayout.h>
#include <qframe.h>
#include <qfile.h>
#include <qlist.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qdir.h>

#include "skymap.h"
#include "geolocation.h"
#include "skyobject.h"
#include "timedialog.h"
#include "locationdialog.h"
#include "finddialog.h"
#include "viewopsdialog.h"

#define TIMER_INTERVAL 10

/** KStars is the base class of the project */
class KStars : public KMainWindow
{
  Q_OBJECT 
  public:
    /** construtor */
    KStars();
    /** destructor */
    ~KStars();

		SkyMap *skymap;
		SkyPoint focus, oldfocus;
		hms LSTh;
		double HourAngle;
		int starCount1, starCount2, starCount3, starCount0;
		KStandardDirs *KDirs;
		QList<GeoLocation> geoList;
		QList<SkyObject> starList;
		QList<SkyObject> messList;
		QList<SkyObject> ngcList;
		QList<SkyObject> icList;
		QList<SkyPoint> Equator;
		QList<SkyPoint> Horizon;
		QList<SkyPoint> clineList;
		QList<QChar> clineModeList;
		QList<SkyObject> cnameList;
		QString TypeName[10];
    QString CurrentPosition;
  	QString SkyColor, StarColor, MessColor, NGCColor, ICColor;
  	QString HSTColor, EqColor, HorzColor, CLineColor, CNameColor;
		bool useAltAz, drawBSC, drawMessier, drawNGC, drawIC;
		bool drawConstellLines, drawConstellNames;
		bool drawEquator, drawHorizon, drawGround, drawEcliptic;
		bool drawSun, drawMoon;
		GeoLocation *geo;
		void initMenuBar();
		void initStatusBar();

	private slots:
		void updateTime( void );
		void mSetTimeToNow();
		void mSetTime();
		void mToggleTimer();
		void mZenith();
		void mFind();
		void mZoomIn();
		void mZoomOut();
		void mTrack();
		void mGeoLocator();
		void mReverseVideo();
		void mViewOps();

	private:
		bool openDataFile( QFile &file, QString filename );
		void readCityData( void );
		void readStarData( void );
		void readNGCData( void );
		void readCLineData( void );
		void readCNameData( void );
		void readURLData( void );
    QString getDateString( QDate d );
		double getJD( QDateTime *t);
		QTime UTtoLST( QDateTime UT, float longitude );
		QTime UTtoGST( QDateTime UT );
		QTime GSTtoLST( QTime GST, float longitude );
		int				  thisSecond, lastSecond, this_msec, last_msec;
		
		QHBoxLayout *iplay;
		QGridLayout *coolay;
		QVBoxLayout *vlay, *tlablay, *timelay, *datelay, *geolay;
		
		QFrame      *infoPanel;
		QLabel			*LT, *UT, *LTLabel, *UTLabel;
		QLabel			*ST, *JD, *STLabel, *JDLabel;
		QLabel			*LTDate, *UTDate;
		QLabel			*CityName, *StateName;
		QLabel			*Long, *Lat, *LongLabel, *LatLabel;
		QTimer			*tmr;
		QDateTime		*LTime, *UTime;
    QTime       *LST;
		SkyObject 	*obj;
    TimeDialog		  *timedialog;
		LocationDialog	*locationdialog;
		FindDialog      *finddialog;
		ViewOpsDialog   *viewopsdialog;
};

#endif
