/***************************************************************************
                          skymap.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Sat Feb 10 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapp.h>
#include <kconfig.h>
#include <klocale.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qcursor.h>
#include <qpointarray.h>
#include <qarray.h>
#include <math.h>
#include <stdlib.h>
#include <stream.h>
#include <unistd.h>
#include "kstars.h"
#include "skymap.h"

SkyMap::SkyMap(QWidget *parent, const char *name ) : QWidget(parent,name) {
	kapp->config()->setGroup( "View" );
	pixelScale = kapp->config()->readNumEntry( "PixelScale", 1024 );
	focus.setRA( kapp->config()->readDoubleNumEntry( "FocusRA", 5.3 ) );
	focus.setDec( kapp->config()->readDoubleNumEntry( "FocusDec", 0.0 ) );
  oldfocus.set( focus.getRA(), focus.getDec() );
	
	setBackgroundColor( QColor( ((KStars *)parent)->SkyColor ) );	
	setBackgroundMode( QWidget::NoBackground );
	setFocusPolicy( QWidget::StrongFocus );
	setMouseTracking (true); //Generate MouseMove events!
	mouseButtonDown = false;

	sky = new QPixmap();
	initPopupMenus();
}

void SkyMap::initPopupMenus( void ) {
	pmStar = new QPopupMenu();
	pmStarTitle = new QLabel( "nothing", pmStar );
	pmStarTitle->setAlignment( AlignCenter );
	pmStar->insertItem( pmStarTitle );
	pmStar->insertSeparator();
	pmStar->insertItem( i18n( "Center on this" ), this, SLOT( slotCenter() ) );
	pmStar->insertSeparator();
	pmStar->insertItem( i18n( "Show 1st-Gen DSS image" ), this, SLOT( slotDSS() ) );
	pmStar->insertItem( i18n( "Show 2nd-Gen DSS image" ), this, SLOT( slotDSS2() ) );

	pmMess = new QPopupMenu();
	pmMessTitle = new QLabel( "Messier Object", pmMess );
	pmMessTitle->setAlignment( AlignCenter );
	pmMessTitle2 = new QLabel( " ", pmMess );
	pmMessTitle2->setAlignment( AlignCenter );
	pmMessType = new QLabel( " ", pmMess );
	pmMessType->setAlignment( AlignCenter );
	pmMess->insertItem( pmMessTitle );
	pmMess->insertItem( pmMessTitle2 );
	pmMess->insertItem( pmMessType );
	pmMess->insertSeparator();
	pmMess->insertItem( i18n( "Center on this" ), this, SLOT( slotCenter() ) );
	pmMess->insertSeparator();
	idMessHST = pmMess->insertItem( i18n( "Show HST image" ), this, SLOT( slotHST() ) );
	pmMess->insertItem( i18n( "Show SEDS image" ), this, SLOT( slotSEDS() ) );
	pmMess->insertItem( i18n( "Show 1st-Gen DSS image" ), this, SLOT( slotDSS() ) );
	pmMess->insertItem( i18n( "Show 2nd-Gen DSS image" ), this, SLOT( slotDSS2() ) );

	pmNGC = new QPopupMenu();
	pmNGCTitle = new QLabel( "NGC/IC Object", pmMess );
	pmNGCTitle->setAlignment( AlignCenter );
	pmNGCTitle2 = new QLabel( " ", pmMess );
	pmNGCTitle2->setAlignment( AlignCenter );
	pmNGCType = new QLabel( " ", pmMess );
	pmNGCType->setAlignment( AlignCenter );
	pmNGC->insertItem( pmNGCTitle );
	pmNGC->insertItem( pmNGCTitle2 );
	pmNGC->insertItem( pmNGCType );
	pmNGC->insertSeparator();
	pmNGC->insertItem( i18n( "Center on this" ), this, SLOT( slotCenter() ) );
	pmNGC->insertSeparator();
	idNGCHST = pmNGC->insertItem( i18n( "Show HST image" ), this, SLOT( slotHST() ) );
	pmNGC->insertItem( i18n( "Show 1st-Gen DSS image" ), this, SLOT( slotDSS() ) );
	pmNGC->insertItem( i18n( "Show 2nd-Gen DSS image" ), this, SLOT( slotDSS2() ) );
}

void SkyMap::setGeometry( int x, int y, int w, int h ) {
	QWidget::setGeometry( x, y, w, h );
	sky->resize( w, h );
}

void SkyMap::setGeometry( const QRect &r ) {
	QWidget::setGeometry( r );
	sky->resize( r.width(), r.height() );
}

void SkyMap::slotCenter( void ) {
	KStars *ksw = (KStars *)parent();
	focus.set( clickedPoint.RA, clickedPoint.Dec );
	focus.RADecToAltAz( LSTh.getD(), ksw->geo->lat() );
	
	oldfocus.set( focus.getRA(), focus.getDec() );
	oldfocus.setAz( focus.getAz() );
	oldfocus.setAlt( focus.getAlt() );

	HourAngle = LSTh.getD() - focus.getRA();
  while ( HourAngle < 0.0 ) HourAngle += 24.0;
	
	update();
}	

void SkyMap::slotDSS( void ) {
	QString cmdLine;
	QString URLprefix( "\"http://archive.stsci.edu/cgi-bin/dss_search?v=1" );
	QString URLsuffix( "&e=J2000&h=15.0&w=15.0&f=gif&c=none&fov=NONE\" &" );
	QString RAString, DecString;
	char decsgn;
	RAString = RAString.sprintf( "&r=%02d+%02d+%02d", clickedPoint.RA.getHour(),
																								 clickedPoint.RA.getMin(),
																								 clickedPoint.RA.getSec() );
	decsgn = '+';
	if (clickedPoint.getDec() < 0.0) decsgn = '-';
	int dd = abs( clickedPoint.Dec.getDeg() );
	int dm = abs( clickedPoint.Dec.getMin() );
	int ds = abs( clickedPoint.Dec.getSec() );

	DecString = DecString.sprintf( "&d=%c%02d+%02d+%02d", decsgn, dd, dm, ds );

	//concat all the segments into the kview command line:
	cmdLine = URLprefix + RAString + DecString + URLsuffix;
	system( "kview " + cmdLine );			
}

void SkyMap::slotDSS2( void ) {
	QString cmdLine;
	QString URLprefix( "\"http://archive.stsci.edu/cgi-bin/dss_search?v=2r" );
	QString URLsuffix( "&e=J2000&h=15.0&w=15.0&f=gif&c=none&fov=NONE\" &" );
	QString RAString, DecString;
	char decsgn;
	RAString = RAString.sprintf( "&r=%02d+%02d+%02d", clickedPoint.RA.getHour(),
																								 clickedPoint.RA.getMin(),
																								 clickedPoint.RA.getSec() );
	decsgn = '+';
	if (clickedPoint.getDec() < 0.0) decsgn = '-';
	int dd = abs( clickedPoint.Dec.getDeg() );
	int dm = abs( clickedPoint.Dec.getMin() );
	int ds = abs( clickedPoint.Dec.getSec() );

	DecString = DecString.sprintf( "&d=%c%02d+%02d+%02d", decsgn, dd, dm, ds );

	//concat all the segments into the kview command line:
	cmdLine = URLprefix + RAString + DecString + URLsuffix;
	system( "kview " + cmdLine );			
}

void SkyMap::slotSEDS( void ) {
	QString cmdLine;
	QString URLprefix( "\"http://www.seds.org/messier/Jpg/m" );
	QString URLsuffix( ".jpg\" &" );
	QString id;
	id = foundObject->name.mid( 2, foundObject->name.length()-2 );
	cmdLine = URLprefix + id + URLsuffix;
	system( "kview " + cmdLine );
}

void SkyMap::slotHST( void ) {
	QString url = foundObject->hstURL;
	
	if (!url.isEmpty() ) {
		system( "kview " + url + " &" );
	}
}
				
void SkyMap::keyReleaseEvent( QKeyEvent *e ) {
	slewing = false;
	update();
}

void SkyMap::keyPressEvent( QKeyEvent *e ) {
	KStars *ksw = (KStars *)parent();
	QString s;

//	if ( ksw->useAltAz ) HADecToAltAz( HourAngle, focus.getDec(), ksw->geo->lat(), Az, Alt );

	switch ( e->key() ) {
		case Key_Left :
			if ( ksw->useAltAz ) {
				focus.setAz( focus.getAz() - 2.0*256/pixelScale );
				focus.AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
			} else {
				focus.setRA( focus.getRA() + 0.1 * 256/pixelScale );
				focus.RADecToAltAz( LSTh.getD(), ksw->geo->lat() );
			}

			slewing = true;
			break;

		case Key_Right :
			if ( ksw->useAltAz ) {
				focus.setAz( focus.getAz() + 2.0*256/pixelScale );
				focus.AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
			} else {
				focus.setRA( focus.getRA() - 0.1 * 256/pixelScale );
				focus.RADecToAltAz( LSTh.getD(), ksw->geo->lat() );
			}


			slewing = true;
			break;

		case Key_Up :
			if ( ksw->useAltAz ) {
				focus.setAlt( focus.getAlt() + 2.0*256/pixelScale );
				if ( focus.getAlt() > 90.0 ) focus.setAlt( 90.0 );
				focus.AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
			} else {
				focus.setDec( focus.getDec() + 2.0 * 256/pixelScale );
				if (focus.getDec() > 90.0) focus.setDec( 90.0 );
				focus.RADecToAltAz( LSTh.getD(), ksw->geo->lat() );
			}

			slewing = true;
			break;

		case Key_Down:
			if ( ksw->useAltAz ) {
				focus.setAlt( focus.getAlt() - 2.0*256/pixelScale );
				if ( focus.getAlt() < -90.0 ) focus.setAlt( -90.0 );
				focus.AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
			} else {
				focus.setDec( focus.getDec() - 2.0 * 256/pixelScale );
				if (focus.getDec() < -90.0) focus.setDec( -90.0 );
				focus.RADecToAltAz( LSTh.getD(), ksw->geo->lat() );
			}

			slewing = true;
			break;

		case Key_Plus:   //Zoom in
		case Key_Equal:
			pixelScale = int( pixelScale*1.4142135623731 ); //sqrt(2)
			if ( pixelScale > 16384 ) pixelScale = 16384;
			break;

		case Key_Minus:  //Zoom out
		case Key_Underscore:
			pixelScale = int( pixelScale/1.4142135623731 ); //sqrt(2)
			if ( pixelScale < 256 ) pixelScale = 256;
			break;
	}

	oldfocus.set( focus.getRA(), focus.getDec() );
	oldfocus.setAz( focus.getAz() );
	oldfocus.setAlt( focus.getAlt() );

	HourAngle = LSTh.getD() - focus.getRA();
  while ( HourAngle < 0.0 ) HourAngle += 24.0;

	update();
}

void SkyMap::mouseMoveEvent( QMouseEvent *e ) {
	KStars *ksw = (KStars *)parent();
	double dx = ( 0.5*width()  - e->x() )/pixelScale;
	double dy = ( 0.5*height() - e->y() )/pixelScale;
	double DmouseRA, DmouseDec;

	//determine RA, Dec of mouse pointer
	dXdYToRaDec( dx, dy, focus, ksw->useAltAz, LSTh.getD(), ksw->geo->lat(), DmouseRA, DmouseDec );

	hms mouseRA( DmouseRA );
	dms mouseDec( DmouseDec );

	if ( mouseButtonDown ) {
		//Update focus such that the sky coords at mouse cursor remain approximately constant
		double dRA = DmouseRA - clickedPoint.getRA();	
		double dDec = DmouseDec - clickedPoint.getDec();

		focus.setRA( focus.getRA() - dRA ); //move focus in opposite direction
		focus.setDec( focus.getDec() - dDec );
		if ( focus.getDec() >90.0 ) focus.setDec( 90.0 );
		if ( focus.getDec() <-90.0 ) focus.setDec( -90.0 );
		focus.RADecToAltAz( LSTh.getD(), ksw->geo->lat() );
		oldfocus.set( focus.getRA(), focus.getDec() );
		oldfocus.setAz( focus.getAz() );
		oldfocus.setAlt( focus.getAlt() );

		HourAngle = LSTh.getD() - focus.getRA();

		//redetermine RA, Dec of mouse pointer, using new focus
		dXdYToRaDec( dx, dy, focus, ksw->useAltAz, LSTh.getD(), ksw->geo->lat(), DmouseRA, DmouseDec );
    clickedPoint.set( DmouseRA, DmouseDec );
    update();

	} else {
	
		QString dummy;
		ksw->CurrentPosition = dummy.sprintf( " %02d:%02d:%02d, %02d:%02d:%02d ",
					mouseRA.getHour(), mouseRA.getMin(),  mouseRA.getSec(),
					mouseDec.getDeg(), mouseDec.getMin(), mouseDec.getSec() );
		ksw->statusBar()->changeItem( ksw->CurrentPosition, 1 );
	}
}

void SkyMap::mouseReleaseEvent( QMouseEvent *e ) {
	mouseButtonDown = false;
	slewing = false;
	update();
}

void SkyMap::mousePressEvent( QMouseEvent *e ) {
	double DmouseRA, DmouseDec;
	KStars *ksw = (KStars *)parent();

	if ( mouseButtonDown == false ) {
		if ( e->button()==LeftButton ) {
			mouseButtonDown = true;
			slewing = true;
		}

		double dx = ( 0.5*width()  - e->x() )/pixelScale;
		double dy = ( 0.5*height() - e->y() )/pixelScale;

		//determine RA, Dec of mouse pointer
		dXdYToRaDec( dx, dy, focus, ksw->useAltAz, LSTh.getD(), ksw->geo->lat(), DmouseRA, DmouseDec );
		clickedPoint.set( DmouseRA, DmouseDec );

//		double dRA_max  = 10*28*1024/3600/pixelScale; //2-pixel distance, in hours
//		double dDec_max = 10*6.5*1024/60/pixelScale; //2-pixel distance, in degrees
	
		double rmin;
		double rstar_min = 10000000.0;
		int istar_min = -1;
	
		//Search stars database for nearby object.		
		for ( unsigned int i=0; i<ksw->starList.count(); ++i ) {
			//test RA and dec to see if this star is roughly nearby
			SkyObject *test = (SkyObject *)ksw->starList.at(i);
			double dRA = test->getRA()-clickedPoint.getRA();
			double dDec = test->getDec()-clickedPoint.getDec();
	//		if ( ( fabs( dRA ) < dRA_max ) && ( fabs( dDec) < dDec_max ) ) {
				//determine angular distance between this object and mouse cursor
				double f = 15.0*cos( DegToRad(test->getDec()) );
			  double r = f*f*dRA*dRA + dDec*dDec; //no need to take sqrt, we just want to ID smallest value.
				if (r < rstar_min) {
					istar_min = i;
					rstar_min = r;
				}
	//		}
		}
		
		rmin = rstar_min;
		int icat = 0;
		
		double rmess_min = 10000000.0;
		int imess_min = -1;
	
		//Search Messier database for nearby object.
		for ( unsigned int i=0; i<ksw->messList.count(); ++i ) {
			//test RA and dec to see if this star is roughly nearby
			SkyObject *test = (SkyObject *)ksw->messList.at(i);
			double dRA = test->getRA()-clickedPoint.getRA();
			double dDec = test->getDec()-clickedPoint.getDec();
				double f = 15.0*cos( DegToRad(test->getDec()) );
			  double r = f*f*dRA*dRA + dDec*dDec; //no need to take sqrt, we just want to ID smallest value.
				if (r < rmess_min) {
					imess_min = i;
					rmess_min = r;
				}
		}
	
		if (rmess_min < rmin) {
			rmin = rmess_min;
			icat = 1;
		}
		
		double rngc_min = 10000000.0;
		int ingc_min = -1;
	
		//Search NGC database for nearby object.
		for ( unsigned int i=0; i<ksw->ngcList.count(); ++i ) {
			//test RA and dec to see if this star is roughly nearby
			SkyObject *test = (SkyObject *)ksw->ngcList.at(i);
			double dRA = test->getRA()-clickedPoint.getRA();
			double dDec = test->getDec()-clickedPoint.getDec();
				double f = 15.0*cos( DegToRad(test->getDec()) );
			  double r = f*f*dRA*dRA + dDec*dDec; //no need to take sqrt, we just want to ID smallest value.
				if (r < rngc_min) {
					ingc_min = i;
					rngc_min = r;
				}
		}
	
		if (rngc_min < rmin) {
			rmin = rngc_min;
			icat = 2;
		}
		
		double ric_min = 10000000.0;
		int iic_min = -1;
	
		//Search NGC database for nearby object.
		for ( unsigned int i=0; i<ksw->icList.count(); ++i ) {
			//test RA and dec to see if this star is roughly nearby
			SkyObject *test = (SkyObject *)ksw->icList.at(i);
			double dRA = test->getRA()-clickedPoint.getRA();
			double dDec = test->getDec()-clickedPoint.getDec();
				double f = 15.0*cos( DegToRad(test->getDec()) );
			  double r = f*f*dRA*dRA + dDec*dDec; //no need to take sqrt, we just want to ID smallest value.
				if (r < ric_min) {
					iic_min = i;
					ric_min = r;
				}
		}

		if (ric_min < rmin) {
			rmin = ric_min;
			icat = 3;
		}
		
		if (rmin < 40.0/pixelScale) {
			switch (icat) {
				case 0: //star
					foundObject = (SkyObject *)ksw->starList.at(istar_min);
	
					switch (e->button()) {
						case LeftButton:
							ksw->statusBar()->changeItem( foundObject->name, 0 );
							break;
						case RightButton:
							pmStarTitle->setText( foundObject->name );
							pmStar->popup( QCursor::pos() );
							break;
					}
					break;
				case 1: //Messier Object
					foundObject = (SkyObject *)ksw->messList.at(imess_min);
	
					switch (e->button()) {
						case LeftButton:
							ksw->statusBar()->changeItem( foundObject->name, 0 );
							break;
						case RightButton:
							pmMessTitle->setText( foundObject->name );
							if (foundObject->longname != QString::null) {
								pmMessTitle2->setText( foundObject->longname );
							} else {
								pmMessTitle2->setText( foundObject->name2 );
							}
							pmMessType->setText( ksw->TypeName[foundObject->type] );
							pmMess->setItemEnabled( idMessHST, !foundObject->hstURL.isEmpty() );
							pmMess->popup( QCursor::pos() );
							break;
					}
					break;
				case 2: //NGC Object
					foundObject = (SkyObject *)ksw->ngcList.at(ingc_min);
	
					switch (e->button()) {
						case LeftButton:
							ksw->statusBar()->changeItem( foundObject->name, 0 );
							break;
						case RightButton:
							pmNGCTitle->setText( foundObject->name );
							pmNGCTitle2->setText( foundObject->longname );
							pmNGCType->setText( ksw->TypeName[foundObject->type] );
							pmNGC->setItemEnabled( idNGCHST, !foundObject->hstURL.isEmpty() );
							pmNGC->popup( QCursor::pos() );
							break;
					}
					break;
				case 3: //IC Object
					foundObject = (SkyObject *)ksw->icList.at(iic_min);
	
					switch (e->button()) {
						case LeftButton:
							ksw->statusBar()->changeItem( foundObject->name, 0 );
							break;
						case RightButton:
							pmNGCTitle->setText( foundObject->name );
							pmNGCTitle2->setText( foundObject->longname );
							pmNGCType->setText( ksw->TypeName[foundObject->type] );
							pmNGC->setItemEnabled( idNGCHST, !foundObject->hstURL.isEmpty() );
							pmNGC->popup( QCursor::pos() );
							break;
					}
					break;
			}
		} else {
			//Empty sky selected.  If left-click, display "nothing" in the status bar.
	    //If right-click, open pmStar with "nothing" as star name.
			foundObject = NULL;
	
			switch (e->button()) {
				case LeftButton:
					ksw->statusBar()->changeItem( i18n( "nothing" ), 0 );
					break;
				case RightButton:
					pmStarTitle->setText( "nothing" );
					pmStar->popup( QCursor::pos() );
					break;
			}
		}
	}
}
	
void SkyMap::mouseDoubleClickEvent( QMouseEvent *e ) {
	double DmouseRA, DmouseDec;
	KStars *ksw = (KStars *)parent();

	double dx = ( 0.5*width()  - e->x() )/pixelScale;
	double dy = ( 0.5*height() - e->y() )/pixelScale;

	//determine RA, Dec of mouse pointer
	dXdYToRaDec( dx, dy, focus, ksw->useAltAz, LSTh.getD(), ksw->geo->lat(), DmouseRA, DmouseDec );
	hms mouseRA( DmouseRA );
	dms mouseDec( DmouseDec );

	QString dummy;
	ksw->CurrentPosition = dummy.sprintf( " %02d:%02d:%02d, %02d:%02d:%02d ",
				mouseRA.getHour(), mouseRA.getMin(),  mouseRA.getSec(),
				mouseDec.getDeg(), mouseDec.getMin(), mouseDec.getSec() );
	ksw->statusBar()->changeItem( ksw->CurrentPosition, 1 );

	if (dx != 0.0 || dy != 0.0) {  //clicked on a new position?
		focus.set( mouseRA, mouseDec );
		focus.RADecToAltAz( LSTh.getD(), ksw->geo->lat() );
		oldfocus.set( mouseRA, mouseDec );
		oldfocus.setAz( focus.getAz() );
		oldfocus.setAlt( focus.getAlt() );

		HourAngle = LSTh.getD() - focus.getRA();
		update();
	}
}

void SkyMap::paintEvent( QPaintEvent *e ) {
	QPainter psky;
	KStars *ksw = (KStars *)parent();

	psky.begin( sky );
	psky.fillRect( 0, 0, width(), height(), QBrush( ksw->SkyColor ) );

	//Draw Equator
	if ( ksw->drawEquator ) {
		psky.setPen( QColor( ksw->EqColor ) );

		QPoint o = getXY( ksw->Equator.at(0), ksw->useAltAz );
		QPoint o1 = o;
		psky.moveTo( o.x(), o.y() );

		for ( unsigned int i=1; i<ksw->Equator.count(); ++i ) {
			o = getXY( ksw->Equator.at(i), ksw->useAltAz );

			int dx = psky.pos().x() - o.x();
		
			if ( abs(dx) < 250 ) {
				psky.lineTo( o.x(), o.y() );
			} else {
				psky.moveTo( o.x(), o.y() );	
			}
		}
		//connect the final segment
		int dx = psky.pos().x() - o1.x();
		if (abs(dx) < 250 ) {
			psky.lineTo( o1.x(), o1.y() );
		} else {
			psky.moveTo( o1.x(), o1.y() );	
		}
  }

	//Draw Constellation Lines
	if ( ksw->drawConstellLines ) {
		psky.setPen( QColor( ksw->CLineColor ) );
		int iLast = 0;

		for ( unsigned int i=0; i < ksw->clineList.count(); ++i ) {
			QPoint o = getXY( ksw->clineList.at(i), ksw->useAltAz );

			if ( ( o.x() >= -1000 && o.x() <= width()+1000 && o.y() >=-1000 && o.y() <=height()+1000 ) &&
					 ( o.x() >= -1000 && o.x() <= width()+1000 && o.y() >=-1000 && o.y() <=height()+1000 ) ) {
				if ( ksw->clineModeList.at(i)->latin1()=='M' ) {
					psky.moveTo( o.x(), o.y() );
				} else if ( ksw->clineModeList.at(i)->latin1()=='D' ) {
					if ( i==iLast+1 ) {
						psky.lineTo( o.x(), o.y() );
					} else {
						psky.moveTo( o.x(), o.y() );
  	      }
				}
				iLast = i;
			}
		}
  }

	//Draw Constellation Names
	//Don't draw names if slewing
	if ( !slewing && ksw->drawConstellNames ) {
		psky.setPen( QColor( ksw->CNameColor ) );
		for ( unsigned int i=0; i < ksw->cnameList.count(); ++i ) {
			QPoint o = getXY( ksw->cnameList.at(i), ksw->useAltAz );
			int dx = 5*ksw->cnameList.at(i)->name.length();
			if (o.x() >= 0 && o.x() <= width() && o.y() >=0 && o.y() <=height() ) {
				psky.drawText( o.x()-dx, o.y(), ksw->cnameList.at(i)->name );
			}
		}
  }
	
	//Draw Stars
  //Only draw bright stars if slewing
	if ( ksw->drawBSC ) {
		psky.setPen( QColor( ksw->SkyColor ) );
		psky.setBrush( QColor( ksw->StarColor ) );

		unsigned int numStars = ksw->starList.count();
		if ( pixelScale < 1200) numStars = ksw->starCount3;
		if ( pixelScale < 800 ) numStars = ksw->starCount2;
		if ( pixelScale < 400 ) numStars = ksw->starCount1;
		if ( slewing ) numStars = ksw->starCount1;

		for ( unsigned int i=0; i < numStars; ++i ) {
			QPoint o = getXY( ksw->starList.at(i), ksw->useAltAz );

			if (o.x() >= 0 && o.x() <= width() && o.y() >=0 && o.y() <=height() ) {
				float mag = ksw->starList.at(i)->mag;
				int size = int( 9.0 - mag ) + 1;
				if ( pixelScale < 400 ) size = int( 7.5 - mag ) + 1;

				drawSymbol( psky, ksw->starList.at(i)->type, o.x(), o.y(), size );
			}
  	}
  }

	//Draw Messier Objects
	if ( /*!slewing &&*/ ksw->drawMessier ) {
		psky.setBrush( QColor( ksw->SkyColor ) );
		psky.setPen( QColor( ksw->MessColor ) );

		for ( unsigned int i=0; i < ksw->messList.count(); ++i ) {
			QPoint o = getXY( ksw->messList.at(i), ksw->useAltAz );
			if (o.x() >= 0 && o.x() <= width() && o.y() >=0 && o.y() <=height() ) {
				float mag = ksw->messList.at(i)->mag;
				//int size = pixelScale*( int( 8.0 - mag ) + 1 )/256;
				int size = pixelScale/256;
				if (size > 16) size = 16;

				drawSymbol( psky, ksw->messList.at(i)->type, o.x(), o.y(), size );
			
				//Draw symbol with new color if it's a Hubble shot
				if ( !ksw->messList.at(i)->hstURL.isEmpty() ) {
					psky.setPen( QColor( ksw->HSTColor ) );
					drawSymbol( psky, ksw->messList.at(i)->type, o.x(), o.y(), size );
					psky.setPen( QColor( ksw->MessColor ) );
				}
			}
		}
  }

  //Draw NGC Objects
  if ( !slewing && ksw->drawNGC ) {
		psky.setBrush( QColor( ksw->SkyColor ) );
		psky.setPen( QColor( ksw->NGCColor ) );
		
		for ( unsigned int i=0; i < ksw->ngcList.count(); ++i ) {
			QPoint o = getXY( ksw->ngcList.at(i), ksw->useAltAz );
			if ( o.x() >= 0 && o.x() <= width() && o.y() >= 0 && o.y() <= height() ) {
				int size = pixelScale/256;
				if (size>8) size = 8;
				
				drawSymbol( psky, ksw->ngcList.at(i)->type, o.x(), o.y(), size );
				
				//Draw symbol with new color if its a Hubble shot
				if ( !ksw->ngcList.at(i)->hstURL.isEmpty() ) {
					psky.setPen( QColor( ksw->HSTColor ) );
					drawSymbol( psky, ksw->ngcList.at(i)->type, o.x(), o.y(), size );
					psky.setPen( QColor( ksw->NGCColor ) );
				}
			}
		}
	}

  //Draw IC Objects
  if ( !slewing && ksw->drawIC ) {
		psky.setBrush( QColor( ksw->SkyColor ) );
		psky.setPen( QColor( ksw->ICColor ) );
		
		for ( unsigned int i=0; i < ksw->icList.count(); ++i ) {
			QPoint o = getXY( ksw->icList.at(i), ksw->useAltAz );
			if ( o.x() >= 0 && o.x() <= width() && o.y() >= 0 && o.y() <= height() ) {
				int size = pixelScale/256;
				if (size>8) size = 8;
				
				drawSymbol( psky, ksw->icList.at(i)->type, o.x(), o.y(), size );
				
				//Draw a red "H" where hubble shots exist
				if ( !ksw->icList.at(i)->hstURL.isEmpty() ) {
					psky.setPen( QColor( ksw->HSTColor ) );
					drawSymbol( psky, ksw->icList.at(i)->type, o.x(), o.y(), size );
					psky.setPen( QColor( ksw->ICColor ) );
				}
			}
		}
	}

	//Draw Horizon

	QPointArray pts;
	QList<QPoint> points;
	int ptsCount = 0;

	if (ksw->drawHorizon || ksw->drawGround) {
		psky.setPen( QColor ( ksw->HorzColor ) );

		psky.setBrush( QColor ( ksw->HorzColor ) );
		pts.resize( 720 );

		for ( unsigned int i=0; i<ksw->Horizon.count(); ++i ) {
			QPoint *o = new QPoint();
			*o = getXY( ksw->Horizon.at(i), ksw->useAltAz );
      bool found = false;

			//Use the QList of points to pre-sort visible horizon points
			if ( o->x() > -100 && o->x() < width() + 100 ) {

				if ( ksw->useAltAz ) {
	        unsigned int j;
					for ( j=0; j<points.count(); ++j ) {
						if ( ksw->useAltAz && o->x() < points.at(j)->x() ) {
							found = true;
							break;
						}
					}
					if ( found ) {
						points.insert( j, o );
					} else {
						points.append( o );
					}
				} else {
					points.append( o );
				}
			}
		}

//	Fill the pts array with sorted horizon points, Draw Horizon Line
		pts.setPoint( 0, points.at(0)->x(), points.at(0)->y() );
		if ( ksw->drawHorizon ) psky.moveTo( points.at(0)->x(), points.at(0)->y() );

		for ( unsigned int i=1; i<points.count(); ++i ) {
			pts.setPoint( i, points.at(i)->x(), points.at(i)->y() );
			if ( ksw->drawHorizon ) {
				if ( !ksw->useAltAz && ( abs( points.at(i)->x() - psky.pos().x() ) > 100 ||
							abs( points.at(i)->y() - psky.pos().y() ) > 250 ) ) {
					psky.moveTo( points.at(i)->x(), points.at(i)->y() );
				} else {
					psky.lineTo( points.at(i)->x(), points.at(i)->y() );
				}
			}
		}

  }

//  Finish the polygon by adding a square bottom edge, offscreen
	if ( ksw->drawGround ) {
		ptsCount = points.count();
		pts.setPoint( ptsCount++, width()+10, height()+10 );
		pts.setPoint( ptsCount++, -10, height()+10 );
		pts.resize( ptsCount );

		psky.drawPolygon( pts );

//  remove all items in points list
		for ( unsigned int i=0; i<points.count(); ++i ) {
			points.remove(i);
		}

//	Draw compass heading labels along horizon
		SkyPoint *c = new SkyPoint;
		QPoint cpoint;
		psky.setPen( QColor ( ksw->SkyColor ) );

		//North
		c->setAz( 359.99 );
		c->setAlt( 0.0 );
		if ( !ksw->useAltAz ) c->AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
		cpoint = getXY( c, ksw->useAltAz );
		cpoint.setY( cpoint.y() + 20 );
		if (cpoint.x() > 0 && cpoint.x() < width() && cpoint.y() > 0 && cpoint.y() < height() ) {
			psky.drawText( cpoint.x(), cpoint.y(), "N" );
		}

		//NorthEast
		c->setAz( 45.0 );
		c->setAlt( 0.0 );
		if ( !ksw->useAltAz ) c->AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
		cpoint = getXY( c, ksw->useAltAz );
		cpoint.setY( cpoint.y() + 20 );
		if (cpoint.x() > 0 && cpoint.x() < width() && cpoint.y() > 0 && cpoint.y() < height() ) {
			psky.drawText( cpoint.x(), cpoint.y(), "NE" );
		}

		//East
		c->setAz( 90.0 );
		c->setAlt( 0.0 );
		if ( !ksw->useAltAz ) c->AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
		cpoint = getXY( c, ksw->useAltAz );
		cpoint.setY( cpoint.y() + 20 );
		if (cpoint.x() > 0 && cpoint.x() < width() && cpoint.y() > 0 && cpoint.y() < height() ) {
			psky.drawText( cpoint.x(), cpoint.y(), "E" );
		}

		//SouthEast
		c->setAz( 135.0 );
		c->setAlt( 0.0 );
		if ( !ksw->useAltAz ) c->AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
		cpoint = getXY( c, ksw->useAltAz );
		cpoint.setY( cpoint.y() + 20 );
		if (cpoint.x() > 0 && cpoint.x() < width() && cpoint.y() > 0 && cpoint.y() < height() ) {
			psky.drawText( cpoint.x(), cpoint.y(), "SE" );
		}

		//South
		c->setAz( 179.99 );
		c->setAlt( 0.0 );
		if ( !ksw->useAltAz ) c->AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
		cpoint = getXY( c, ksw->useAltAz );
		cpoint.setY( cpoint.y() + 20 );
		if (cpoint.x() > 0 && cpoint.x() < width() && cpoint.y() > 0 && cpoint.y() < height() ) {
			psky.drawText( cpoint.x(), cpoint.y(), "S" );
		}

		//SouthWest
		c->setAz( 225.0 );
		c->setAlt( 0.0 );
		if ( !ksw->useAltAz ) c->AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
		cpoint = getXY( c, ksw->useAltAz );
		cpoint.setY( cpoint.y() + 20 );
		if (cpoint.x() > 0 && cpoint.x() < width() && cpoint.y() > 0 && cpoint.y() < height() ) {
			psky.drawText( cpoint.x(), cpoint.y(), "SW" );
		}

		//West
		c->setAz( 270.0 );
		c->setAlt( 0.0 );
		if ( !ksw->useAltAz ) c->AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
		cpoint = getXY( c, ksw->useAltAz );
		cpoint.setY( cpoint.y() + 20 );
		if (cpoint.x() > 0 && cpoint.x() < width() && cpoint.y() > 0 && cpoint.y() < height() ) {
			psky.drawText( cpoint.x(), cpoint.y(), "W" );
		}

		//NorthWest
		c->setAz( 315.0 );
		c->setAlt( 0.0 );
		if ( !ksw->useAltAz ) c->AltAzToRADec( LSTh.getD(), ksw->geo->lat() );
		cpoint = getXY( c, ksw->useAltAz );
		cpoint.setY( cpoint.y() + 20 );
		if (cpoint.x() > 0 && cpoint.x() < width() && cpoint.y() > 0 && cpoint.y() < height() ) {
			psky.drawText( cpoint.x(), cpoint.y(), "NW" );
		}

		delete c;
	}

	psky.end();
	bitBlt( this, 0, 0, sky );
}

void SkyMap::drawSymbol( QPainter &psky, int type, int x, int y, int size ) {
	int x1 = x - size/2;
	int x2 = x + size/2;
	int y1 = y - size/2;
	int y2 = y + size/2;

	int xa = x - size/4;
	int xb = x + size/4;
	int ya = y - size/4;
	int yb = y + size/4;

	switch (type) {
		case 0: //star
		case 1: //double star
			if (size > 1 ) {
				psky.drawEllipse( x1, y1, size, size );
			} else if (size==1) {
				psky.drawPoint( x, y );
			}
			break;
		case 2: //Planet
			break;
		case 3: //Open cluster
			psky.drawEllipse( xa, y1, 2, 2 ); // draw circle of points
			psky.drawEllipse( xb, y1, 2, 2 );
			psky.drawEllipse( xa, y2, 2, 2 );
			psky.drawEllipse( xb, y2, 2, 2 );
			psky.drawEllipse( x1, ya, 2, 2 );
			psky.drawEllipse( x1, yb, 2, 2 );
			psky.drawEllipse( x2, ya, 2, 2 );
			psky.drawEllipse( x2, yb, 2, 2 );
			break;
		case 4: //Globular Cluster
			if (size<2) size = 2;
			psky.drawEllipse( x1, y1, size, size );
			psky.moveTo( x, y1 );
			psky.lineTo( x, y2 );
			psky.moveTo( x1, y );
			psky.lineTo( x2, y );
			break;
		case 5: //Gaseous Nebula
			if (size <2) size = 2;
			psky.drawLine( x1, y1, x2, y1 );
			psky.drawLine( x2, y1, x2, y2 );
			psky.drawLine( x2, y2, x1, y2 );
			psky.drawLine( x1, y2, x1, y1 );
			break;
		case 6: //Planetary Nebula
			if (size<2) size = 2;
			psky.drawEllipse( x1, y1, size, size );
			psky.moveTo( x, y1 );
			psky.lineTo( x, y1 - size/2 );
			psky.moveTo( x, y2 );
			psky.lineTo( x, y2 + size/2 );
			psky.moveTo( x1, y );
			psky.lineTo( x1 - size/2, y );
			psky.moveTo( x2, y );
			psky.lineTo( x2 + size/2, y );
			break;
		case 7: //Supernova remnant
			if (size<2) size = 2;
			psky.moveTo( x, y1 );
			psky.lineTo( x2, y );
			psky.lineTo( x, y2 );
			psky.lineTo( x1, y );
			psky.lineTo( x, y1 );
			break;
		case 8: //Galaxy
			if ( size>1 ) {
				psky.drawEllipse( x-size, y1, 2*size, size );
			} else {
				psky.drawPoint( x, y );
			}
			break;
	}
}

QPoint SkyMap::getXY( SkyObject *o, bool Horiz ) {
	QPoint p;
	double X, Y, X0, Y0;
	double dX, sindX, cosdX, sinY, cosY, sinY0, cosY0;

	if ( Horiz ) {
		X0 = focus.getAz();
		Y0 = focus.getAlt();

		X = o->getAz();
		Y = o->getAlt();

		if ( X0 > 270.0 && X < 90.0 ) {
			dX = 360.0 + X0 - X;
		} else {
			dX = X0 - X;
		}
		SinCos( Y0, sinY0, cosY0 );

  } else {
		if (focus.getRA() > 18.0 && o->getRA() < 6.0) {
			dX = HourToDeg( o->getRA() + 24.0 - focus.getRA() );
		} else {
			dX = HourToDeg( o->getRA() - focus.getRA() );
	  }
    Y = o->getDec();
		SinCos( focus.getDec(), sinY0, cosY0 );
  }

	//Convert dX, Y coords to screen pixel coords.
	SinCos( dX, sindX, cosdX );
	SinCos( Y, sinY, cosY );
	
	double c = sinY0*sinY + cosY0*cosY*cosdX;

	if ( c < 0.0 ) { //Object is on "back side" of the celestial sphere; don't plot it.
		p.setX( -100000 );
		p.setY( -100000 );
		return p;
	}

	double k = sqrt( 2.0/( 1 + c ) );

	p.setX( int( 0.5*width()  - pixelScale*k*cosY*sindX ) );
	p.setY( int( 0.5*height() - pixelScale*k*( cosY0*sinY - sinY0*cosY*cosdX ) ) );

	return p;
}

QPoint SkyMap::getXY( SkyPoint *o, bool Horiz ) {
	QPoint p;
	double X, Y, X0, Y0;
	double dX, sindX, cosdX, sinY, cosY, sinY0, cosY0;

	if ( Horiz ) {
		X0 = focus.getAz();
		Y0 = focus.getAlt();

		//convert object RA, Dec to Az, Alt (X, Y)
		X = o->getAz();
		Y = o->getAlt();

		if ( X0 > 270.0 && X < 90.0 ) {
			dX = 360.0 + X0 - X;
		} else {
			dX = X0 - X;
		}
		SinCos( Y0, sinY0, cosY0 );

  } else {
		if (focus.getRA() > 18.0 && o->getRA() < 6.0) {
			dX = HourToDeg( o->getRA() + 24.0 - focus.getRA() );
		} else {
			dX = HourToDeg( o->getRA() - focus.getRA() );
	  }
    Y = o->getDec();
		SinCos( focus.getDec(), sinY0, cosY0 );
  }

	//Convert dX, Y coords to screen pixel coords.
	SinCos( dX, sindX, cosdX );
	SinCos( Y, sinY, cosY );
	
	double c = sinY0*sinY + cosY0*cosY*cosdX;

	if ( c < 0.0 ) { //Object is on "back side" of the celestial sphere; don't plot it.
		p.setX( -100000 );
		p.setY( -100000 );
		return p;
	}

	double k = sqrt( 2.0/( 1 + c ) );

	p.setX( int( 0.5*width()  - pixelScale*k*cosY*sindX ) );
	p.setY( int( 0.5*height() - pixelScale*k*( cosY0*sinY - sinY0*cosY*cosdX ) ) );

	return p;
}

