/***************************************************************************
                          skymap.h  -  K Desktop Planetarium
                             -------------------
    begin                : Sat Feb 10 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/




#ifndef SKYMAP_H
#define SKYMAP_H

#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpoint.h>
#include "skyobject.h"
#include "hms.h"
#include "skypoint.h"

class QPopupMenu;
class QLabel;

/**Class for the sky display, derived from QWidget.  This widget is just a canvas
	*on which the sky is painted.  It's the main widget for KStars.  The widget also
	* handles user interaction (both mouse and keyboard).
	*
	*@short Canvas widget for displaying the sky bitmap; also handles user input.
  *@author Jason Harris <jharris@30doradus.org>
  */

class SkyMap : public QWidget  {
   Q_OBJECT
public:
/**
	*Create an instance of SkyMap.
	*/
	SkyMap(QWidget *parent=0, const char *name=0);
	~SkyMap() {};

	QPixmap 	*sky;
  SkyPoint  focus, oldfocus, clickedPoint;
	SkyObject *foundObject;
  hms 			LSTh;
  double 		HourAngle;
  int 			pixelScale;
	int idMessHST, idNGCHST;
	bool slewing;

public slots:
	virtual void setGeometry( int x, int y, int w, int h );
  virtual void setGeometry( const QRect &r );
	void slotCenter( void );
	void slotDSS( void );
	void slotDSS2( void );
	void slotSEDS( void );
	void slotHST( void );
	
protected:
	virtual void paintEvent( QPaintEvent *e );
	virtual void keyPressEvent( QKeyEvent *e );
	virtual void keyReleaseEvent( QKeyEvent *e );
	virtual void mousePressEvent( QMouseEvent *e );
	virtual void mouseReleaseEvent( QMouseEvent *e );
	virtual void mouseDoubleClickEvent( QMouseEvent *e );
	virtual void mouseMoveEvent( QMouseEvent *e );

private:
	QPoint getXY( SkyObject *o, bool Horiz );
	QPoint getXY( SkyPoint *o, bool Horiz );
	QPopupMenu *pmStar, *pmMess, *pmNGC;
	QLabel *pmStarTitle, *pmMessTitle, *pmMessTitle2, *pmNGCTitle, *pmNGCTitle2;
	QLabel *pmMessType, *pmNGCType;
	void initPopupMenus( void );
	void drawSymbol( QPainter &p, int type, int x, int y, int size );
	bool mouseButtonDown;
};

#endif
