/***************************************************************************
                          skypoint.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Sun Feb 11 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//All functions except destructor are defined inline in the header (skypoint.h)

#include "dms.h"
#include "kstars.h"
#include "skypoint.h"

SkyPoint::~SkyPoint(){
}

void SkyPoint::RADecToAltAz( double LSTh, double lat ) {
	double AltD, AzD;
	double sindec, cosdec, sinlat, coslat, HourAngle, sinHA, cosHA;
  double sinAlt, cosAlt;

	HourAngle = LSTh - getRA();

	SinCos( lat, sinlat, coslat );
	SinCos( getDec(), sindec, cosdec );
	SinCos( HourToDeg( HourAngle ), sinHA, cosHA );

	sinAlt = sindec*sinlat + cosdec*coslat*cosHA;
	AltD = asin( sinAlt );
	cosAlt = cos( AltD );

	AzD = acos( ( sindec - sinlat*sinAlt )/( coslat*cosAlt ) );
	AzD = RadToDeg( AzD );
	if ( sinHA > 0.0 ) AzD = 360.0 - AzD; // resolve acos() ambiguity

	setAz( AzD );
	setAlt( RadToDeg( AltD ) );

}

void SkyPoint::AltAzToRADec( double LSTh, double lat ) {
	double HA, DecD;
	double sinlat, coslat, sinAlt, cosAlt, sinAz, cosAz;
  double sinDec, cosDec;

	SinCos( lat, sinlat, coslat );
	SinCos( getAlt(), sinAlt, cosAlt );
	SinCos(  getAz(),  sinAz,  cosAz );

	sinDec = sinAlt*sinlat + cosAlt*coslat*cosAz;
	DecD = asin( sinDec );
	cosDec = cos( DecD );

	HA = acos( ( sinAlt - sinlat*sinDec )/( coslat*cosDec ) );
	HA = RadToHour( HA );
	setDec( RadToDeg( DecD ) );
	
	if ( sinAz > 0.0 ) HA = 24.0 - HA; // resolve acos() ambiguity	
	setRA( LSTh - HA );
}

