/***************************************************************************
                          viewopsdialog.cpp  -  K Desktop Planetarium
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2001 by Jason Harris
    email                : jharris@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <klocale.h>
#include <qlayout.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qframe.h>
#include <qbutton.h>
#include <qcolor.h>
#include <qfont.h>
#include <qcolordialog.h>

#include "kstars.h"
#include "viewopsdialog.h"

ViewOpsDialog::ViewOpsDialog( QWidget *parent )
    : KDialogBase( KDialogBase::Plain, i18n( "Display Options" ), Ok|Cancel, Ok, parent ) {
	QFrame *page = plainPage();
	KStars *ksw = (KStars *)parent;

	// Layout manager for entire dialog
	hlay = new QHBoxLayout( page, 2, 2 );
	hlay->setSpacing( 2 );
	hlay->setMargin( 4 );

	DisplayBox = new QGroupBox( page, "DisplayBox" );
  DisplayBox->setMinimumWidth( 230 );
  DisplayBox->setTitle( i18n( "Display Options" ) );

	DisplayBoxLayout = new QVBoxLayout( DisplayBox );
	DisplayBoxLayout->setSpacing( 6 );
	DisplayBoxLayout->setMargin( 11 );

	DisplayTabs = new QTabWidget( DisplayBox, "DisplayTabs" );
	DisplayTabs->move( 10, 24 );
	DisplayTabs->setMinimumSize( 220, 300 );

	CoordsGroup = new QButtonGroup( 1, Qt::Vertical, "Coordinate system", DisplayBox );
	
	EquatRadio = new QRadioButton( i18n( "Equatorial" ), CoordsGroup );
	EquatRadio->setChecked( !ksw->useAltAz );

	AltAzRadio = new QRadioButton( i18n( "Altitude/Azimuth" ), CoordsGroup );
	AltAzRadio->setChecked( ksw->useAltAz );

	DisplayBoxLayout->addSpacing( 10 );
	DisplayBoxLayout->addWidget( DisplayTabs );
	DisplayBoxLayout->addWidget( CoordsGroup );

//Construct Catalogs tab:
	CatalogTab = new QWidget( DisplayTabs, "CatalogTab" );
	vlay1 = new QVBoxLayout( CatalogTab );
	vlay1->setSpacing( 6 );
	vlay1->setMargin( 11 );

	showBSC = new QCheckBox( i18n( "Yale Bright Star Catalog" ), CatalogTab );
  QFont stdFont(  showBSC->font() );
//  stdFont.setFamily( "helvetica" );
//  stdFont.setPointSize( 10 );
//  stdFont.setBold( FALSE );
  showBSC->setFont( stdFont );
  showBSC->setChecked( ksw->drawBSC );

	showMessier = new QCheckBox( i18n( "Messier Objects" ), CatalogTab );
  showMessier->setFont( stdFont );
  showMessier->setChecked( ksw->drawMessier );

	showNGC = new QCheckBox( i18n( "NGC Catalog" ), CatalogTab );
  showNGC->setFont( stdFont );
	showNGC->setChecked( ksw->drawNGC );

	showIC = new QCheckBox( i18n( "IC Catalog" ), CatalogTab );
  showIC->setFont( stdFont );
	showIC->setChecked( ksw->drawIC );

	QSpacerItem *spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );

  vlay1->addWidget( showBSC );
	vlay1->addWidget( showMessier );
	vlay1->addWidget( showNGC );
	vlay1->addWidget( showIC );
	vlay1->addItem( spacer );

	DisplayTabs->insertTab( CatalogTab, i18n( "Catalogs" ) );

//Construct Guides tab:
	GuideTab = new QWidget( DisplayTabs, "GuideTab" );
	vlay2 = new QVBoxLayout( GuideTab );
	vlay2->setSpacing( 6 );
	vlay2->setMargin( 11 );

	showConstellLines = new QCheckBox( i18n( "Constellation Lines" ), GuideTab );
  showConstellLines->setFont( stdFont );
  showConstellLines->setChecked( ksw->drawConstellLines );

	showConstellNames = new QCheckBox( i18n( "Constellation Names" ), GuideTab );
  showConstellNames->setFont( stdFont );
  showConstellNames->setChecked( ksw->drawConstellNames );

	showEquator = new QCheckBox( i18n( "Celestial Equator" ), GuideTab );
  showEquator->setFont( stdFont );
  showEquator->setChecked( ksw->drawEquator );

	showEcliptic = new QCheckBox( i18n( "Ecliptic" ), GuideTab );
  showEcliptic->setFont( stdFont );
//  showEcliptic->setChecked( ksw->drawEcliptic );
  showEcliptic->setChecked( false );

	showHorizon = new QCheckBox( i18n( "Horizon (line)" ), GuideTab );
  showHorizon->setFont( stdFont );
  showHorizon->setChecked( ksw->drawHorizon );

	showGround = new QCheckBox( i18n( "Opaque Ground" ), GuideTab );
  showGround->setFont( stdFont );
  showGround->setChecked( ksw->drawGround );

	QSpacerItem *spacer2 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );

  vlay2->addWidget( showConstellLines );
	vlay2->addWidget( showConstellNames );
	vlay2->addWidget( showEquator );
	vlay2->addWidget( showEcliptic );
	vlay2->addWidget( showHorizon );
	vlay2->addWidget( showGround );
	vlay2->addItem( spacer2 );

	DisplayTabs->insertTab( GuideTab, i18n( "Guides" ) );

//Construct Planets tab:
	PlanetTab = new QWidget( DisplayTabs, "PlanetTab" );
	vlay3 = new QVBoxLayout( PlanetTab );
	vlay3->setSpacing( 6 );
	vlay3->setMargin( 11 );

	showSun = new QCheckBox( i18n( "The Sun" ), PlanetTab );
  showSun->setFont( stdFont );
  showSun->setChecked( ksw->drawSun );
  showSun->setChecked( false );

	showMoon = new QCheckBox( i18n( "The Moon" ), PlanetTab );
  showMoon->setFont( stdFont );
  showMoon->setChecked( ksw->drawMoon );
  showMoon->setChecked( false );

	QSpacerItem *spacer3 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );

  vlay3->addWidget( showSun );
	vlay3->addWidget( showMoon );
  vlay3->addItem( spacer3 );

	DisplayTabs->insertTab( PlanetTab, i18n( "Planets" ) );

//Construct the Color palette box
  ColorBox = new QGroupBox( page, "ColorBox" );
  ColorBox->setTitle( i18n( "Color Options" ) );

	ColorBoxLayout = new QVBoxLayout( ColorBox );
	ColorBoxLayout->setSpacing( 6 );
	ColorBoxLayout->setMargin( 11 );

	ColorPaletteLayout = new QGridLayout( 11, 2, 2 );

	ShowCSky = new QFrame( ColorBox );
	ShowCSky->setFixedSize( 16, 16 );
	ShowCSky->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCSky->setBackgroundColor( QColor( ksw->SkyColor ) );

	ShowCStar = new QFrame( ColorBox );
	ShowCStar->setFixedSize( 16, 16 );
	ShowCStar->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCStar->setBackgroundColor( QColor( ksw->StarColor ) );

	ShowCMess = new QFrame( ColorBox );
	ShowCMess->setFixedSize( 16, 16 );
	ShowCMess->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCMess->setBackgroundColor( QColor( ksw->MessColor ) );

	ShowCNGC = new QFrame( ColorBox );
	ShowCNGC->setFixedSize( 16, 16 );
	ShowCNGC->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCNGC->setBackgroundColor( QColor( ksw->NGCColor ) );

	ShowCIC = new QFrame( ColorBox );
	ShowCIC->setFixedSize( 16, 16 );
	ShowCIC->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCIC->setBackgroundColor( QColor( ksw->ICColor ) );

	ShowCEquator = new QFrame( ColorBox );
	ShowCEquator->setFixedSize( 16, 16 );
	ShowCEquator->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCEquator->setBackgroundColor( QColor( ksw->EqColor ) );

	ShowCHorizon = new QFrame( ColorBox );
	ShowCHorizon->setFixedSize( 16, 16 );
	ShowCHorizon->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCHorizon->setBackgroundColor( QColor( ksw->HorzColor ) );

	ShowCConstLine = new QFrame( ColorBox );
	ShowCConstLine->setFixedSize( 16, 16 );
	ShowCConstLine->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCConstLine->setBackgroundColor( QColor( ksw->CLineColor ) );

	ShowCConstText = new QFrame( ColorBox );
	ShowCConstText->setFixedSize( 16, 16 );
	ShowCConstText->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCConstText->setBackgroundColor( QColor( ksw->CNameColor ) );

	ShowCHST = new QFrame( ColorBox );
	ShowCHST->setFixedSize( 16, 16 );
	ShowCHST->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	ShowCHST->setBackgroundColor( QColor( ksw->HSTColor ) );

	ChangeCSky = new QPushButton( i18n( "Sky" ), ColorBox );
	ChangeCStar = new QPushButton( i18n( "Star" ), ColorBox );
	ChangeCMess = new QPushButton( i18n( "Messier Object" ), ColorBox );
	ChangeCNGC = new QPushButton( i18n( "NGC Object" ), ColorBox );
	ChangeCIC = new QPushButton( i18n( "IC Object" ), ColorBox );
	ChangeCEquator = new QPushButton( i18n( "Equator" ), ColorBox );
	ChangeCHorizon = new QPushButton( i18n( "Horizon" ), ColorBox );
	ChangeCConstLine = new QPushButton( i18n( "Constell. Line" ), ColorBox );
	ChangeCConstText = new QPushButton( i18n( "Constell. Name" ), ColorBox );
	ChangeCHST = new QPushButton( i18n( "HST Image" ), ColorBox );

	NightColors = new QPushButton( i18n( "Night Vision"), ColorBox );
	ResetColors = new QPushButton( i18n( "Default Colors"), ColorBox );
	
//Pack the widgets into the layouts
	ColorBoxLayout->addSpacing( 6 );
	ColorBoxLayout->addLayout( ColorPaletteLayout, 0 );
//	ColorBoxLayout->addWidget( NightColors );
//	ColorBoxLayout->addWidget( ResetColors );


	ColorPaletteLayout->addWidget( ShowCSky, 0, 0 );
	ColorPaletteLayout->addWidget( ShowCStar, 1, 0 );
	ColorPaletteLayout->addWidget( ShowCMess, 2, 0 );
	ColorPaletteLayout->addWidget( ShowCNGC, 3, 0 );
	ColorPaletteLayout->addWidget( ShowCIC, 4, 0 );
	ColorPaletteLayout->addWidget( ShowCEquator, 5, 0 );
	ColorPaletteLayout->addWidget( ShowCHorizon, 6, 0 );
	ColorPaletteLayout->addWidget( ShowCConstLine, 7, 0 );
	ColorPaletteLayout->addWidget( ShowCConstText, 8, 0 );
	ColorPaletteLayout->addWidget( ShowCHST, 9, 0 );
	ColorPaletteLayout->addWidget( NightColors, 10, 0 );

	ColorPaletteLayout->addWidget( ChangeCSky, 0, 1 );
	ColorPaletteLayout->addWidget( ChangeCStar, 1, 1 );
	ColorPaletteLayout->addWidget( ChangeCMess, 2, 1 );
	ColorPaletteLayout->addWidget( ChangeCNGC, 3, 1 );
	ColorPaletteLayout->addWidget( ChangeCIC, 4, 1 );
	ColorPaletteLayout->addWidget( ChangeCEquator, 5, 1 );
	ColorPaletteLayout->addWidget( ChangeCHorizon, 6, 1 );
	ColorPaletteLayout->addWidget( ChangeCConstLine, 7, 1 );
	ColorPaletteLayout->addWidget( ChangeCConstText, 8, 1 );
	ColorPaletteLayout->addWidget( ChangeCHST, 9, 1 );
	ColorPaletteLayout->addWidget( ResetColors, 10, 1 );

	hlay->addWidget( DisplayBox );
	hlay->addSpacing( 12 );
	hlay->addWidget( ColorBox );
	hlay->activate();

  connect( this, SIGNAL( okClicked() ), this, SLOT( accept() ) ) ;
  connect( this, SIGNAL( cancelClicked() ), this, SLOT( reject() ) );
  connect( ChangeCSky, SIGNAL( clicked() ), this, SLOT( newSkyColor() ) );
  connect( ChangeCStar, SIGNAL( clicked() ), this, SLOT( newStarColor() ) );
  connect( ChangeCMess, SIGNAL( clicked() ), this, SLOT( newMessColor() ) );
  connect( ChangeCNGC, SIGNAL( clicked() ), this, SLOT( newNGCColor() ) );
  connect( ChangeCIC, SIGNAL( clicked() ), this, SLOT( newICColor() ) );
  connect( ChangeCEquator, SIGNAL( clicked() ), this, SLOT( newEqColor() ) );
  connect( ChangeCHorizon, SIGNAL( clicked() ), this, SLOT( newHorzColor() ) );
  connect( ChangeCConstLine, SIGNAL( clicked() ), this, SLOT( newCLineColor() ) );
  connect( ChangeCConstText, SIGNAL( clicked() ), this, SLOT( newCNameColor() ) );
  connect( ChangeCHST, SIGNAL( clicked() ), this, SLOT( newHSTColor() ) );
	connect( NightColors, SIGNAL( clicked() ), this, SLOT( redColors() ) );
	connect( ResetColors, SIGNAL( clicked() ), this, SLOT( defaultColors() ) );
	connect( showBSC, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showMessier, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showNGC, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showIC, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showConstellLines, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showConstellNames, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showEquator, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showHorizon, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( showGround, SIGNAL( clicked() ), this, SLOT( updateDisplay() ) );
	connect( EquatRadio, SIGNAL( clicked() ), this, SLOT( changeCoordSys() ) );
	connect( AltAzRadio, SIGNAL( clicked() ), this, SLOT( changeCoordSys() ) );
}

ViewOpsDialog::~ViewOpsDialog(){
}

void ViewOpsDialog::newSkyColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->SkyColor = QColorDialog::getColor( QColor( ksw->SkyColor ) ).name();
	ShowCSky->setBackgroundColor( QColor( ksw->SkyColor ) );
}	

void ViewOpsDialog::newStarColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->StarColor = QColorDialog::getColor( QColor( ksw->StarColor ) ).name();
	ShowCStar->setBackgroundColor( QColor( ksw->StarColor ) );
}	

void ViewOpsDialog::newMessColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->MessColor = QColorDialog::getColor( QColor( ksw->MessColor ) ).name();
	ShowCMess->setBackgroundColor( QColor( ksw->MessColor ) );
}	

void ViewOpsDialog::newNGCColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->NGCColor = QColorDialog::getColor( QColor( ksw->NGCColor ) ).name();
	ShowCNGC->setBackgroundColor( QColor( ksw->NGCColor ) );
}	

void ViewOpsDialog::newICColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->ICColor = QColorDialog::getColor( QColor( ksw->ICColor ) ).name();
	ShowCIC->setBackgroundColor( QColor( ksw->ICColor ) );
}	

void ViewOpsDialog::newHSTColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->HSTColor = QColorDialog::getColor( QColor( ksw->HSTColor ) ).name();
	ShowCHST->setBackgroundColor( QColor( ksw->HSTColor ) );
}	

void ViewOpsDialog::newEqColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->EqColor = QColorDialog::getColor( QColor( ksw->EqColor ) ).name();
	ShowCEquator->setBackgroundColor( QColor( ksw->EqColor ) );
}	

void ViewOpsDialog::newHorzColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->HorzColor = QColorDialog::getColor( QColor( ksw->HorzColor ) ).name();
	ShowCHorizon->setBackgroundColor( QColor( ksw->HorzColor ) );
}	

void ViewOpsDialog::newCLineColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->CLineColor = QColorDialog::getColor( QColor( ksw->CLineColor ) ).name();
	ShowCConstLine->setBackgroundColor( QColor( ksw->CLineColor ) );
}	

void ViewOpsDialog::newCNameColor( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->CNameColor = QColorDialog::getColor( QColor( ksw->CNameColor ) ).name();
	ShowCConstText->setBackgroundColor( QColor( ksw->CNameColor ) );
}	

void ViewOpsDialog::defaultColors( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->SkyColor = "#002";
	ksw->StarColor = "#FFF";
  ksw->EqColor = "#FFF";
  ksw->HorzColor = "#5A3";
	ksw->MessColor = "#0F0";
	ksw->NGCColor = "#066";
	ksw->ICColor = "#439";
	ksw->CLineColor = "#555";
	ksw->CNameColor = "#AA7";
	ksw->HSTColor = "#A00";
	
	ShowCSky->setBackgroundColor( QColor( ksw->SkyColor ) );
	ShowCStar->setBackgroundColor( QColor( ksw->StarColor ) );
	ShowCMess->setBackgroundColor( QColor( ksw->MessColor ) );
	ShowCNGC->setBackgroundColor( QColor( ksw->NGCColor ) );
	ShowCIC->setBackgroundColor( QColor( ksw->ICColor ) );
	ShowCEquator->setBackgroundColor( QColor( ksw->EqColor ) );
	ShowCHorizon->setBackgroundColor( QColor( ksw->HorzColor ) );
	ShowCConstLine->setBackgroundColor( QColor( ksw->CLineColor ) );
	ShowCConstText->setBackgroundColor( QColor( ksw->CNameColor ) );
	ShowCHST->setBackgroundColor( QColor( ksw->HSTColor ) );
	ksw->skymap->update();	
}

void ViewOpsDialog::redColors( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->SkyColor = "#000";
	ksw->StarColor = "#F00";
  ksw->EqColor = "#A00";
  ksw->HorzColor = "#933";
	ksw->MessColor = "#C09";
	ksw->NGCColor = "#C09";
	ksw->ICColor = "#C09";
	ksw->CLineColor = "#800";
	ksw->CNameColor = "#900";
	ksw->HSTColor = "#A00";
	
	ShowCSky->setBackgroundColor( QColor( ksw->SkyColor ) );
	ShowCStar->setBackgroundColor( QColor( ksw->StarColor ) );
	ShowCMess->setBackgroundColor( QColor( ksw->MessColor ) );
	ShowCNGC->setBackgroundColor( QColor( ksw->NGCColor ) );
	ShowCIC->setBackgroundColor( QColor( ksw->ICColor ) );
	ShowCEquator->setBackgroundColor( QColor( ksw->EqColor ) );
	ShowCHorizon->setBackgroundColor( QColor( ksw->HorzColor ) );
	ShowCConstLine->setBackgroundColor( QColor( ksw->CLineColor ) );
	ShowCConstText->setBackgroundColor( QColor( ksw->CNameColor ) );
	ShowCHST->setBackgroundColor( QColor( ksw->HSTColor ) );
	ksw->skymap->update();	
}

void ViewOpsDialog::updateDisplay( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->drawBSC = showBSC->isChecked();
	ksw->drawMessier = showMessier->isChecked();
	ksw->drawNGC = showNGC->isChecked();
	ksw->drawIC = showIC->isChecked();
	ksw->drawConstellLines = showConstellLines->isChecked();
	ksw->drawConstellNames = showConstellNames->isChecked();
	ksw->drawEquator = showEquator->isChecked();
	ksw->drawHorizon = showHorizon->isChecked();
	ksw->drawGround = showGround->isChecked();
	ksw->skymap->update();
}

void ViewOpsDialog::changeCoordSys( void ) {
	KStars *ksw = (KStars *)parent();
	ksw->useAltAz = AltAzRadio->isChecked();
	showGround->setEnabled( AltAzRadio->isChecked() );

	if ( EquatRadio->isChecked() ) {
		showGround->setChecked( false );
	}

	ksw->drawGround = showGround->isChecked();
	ksw->skymap->update();
}
